/*
 *	API Sniffer
 *	Copyright (c) 1992, 1993 by WinSoft
 *
 *				M O D I F I C A T I O N      H I S T O R Y
 *
 *	Rev.	Description										Who		Date
 *	------------------------------------------------------------------------
 *	1.0		Created.										DLC		11/92
 *	1.0a	Fixed a nasty bug in OpenFile() which would		DLC		04/93
 *			cause closing a DOS VM to crash Windows.
 *			Added support for mciSendString() and
 *			mciSendCommand().
 *
 *	1.0b	Added support for GetProcAddress(),				DLC		04/93
 *			OpenPathName (undocumented).
 *
 *	This sample program is designed to demonstrate the capability of the TRAPFUNC
 *	library.  While this API SNIFFER lacks the structure of an all-purpose tool,
 *	it does demonstrate that it is possible, using the TRAPFUNC library, to build
 *	such a tool.
 *
 *  Windows 3.1 SDK functions from KERNEL, USER, and GDI are trapped by this
 *	application.  It is also possible to trap entrypoints into Win 3.1 drivers
 *	or third party DLLs.
 *
 *	APISNIFF subclasses 13 WinSDK 3.1 functions.  All calls made system-wide to any of
 *	the subclassed functions are first sent to the user-defined hook function.  Within
 *	this function, the HOOK is temporarily disabled, then the parameters are output to
 *	the APISNIFF main-window.  The system function call is then called, and lastly, before
 *	returning, the HOOK is re-enabled.  The technique used is similar to that of
 *	sub-classing a window.
 *
 *	User-defined functions were used for each system function subclassed.  This approach
 *	was chosen to reduce the complexity of this sample program.  It is possible and it has
 *	been demonstrated that a central callback function may be defined which can be used
 *	by all functions subclassed.
 */

/************************************************************************\
 |                      INCLUDE FILE DEFINITIONS                        |
\************************************************************************/
#include <windows.h>
#include <mmsystem.h>
#include <dos.h>
#include <mem.h>
#include <string.h>
#include <dir.h>
#include <stdlib.h>
#include "trapfunc.h"
#include "apisniff.h"

/************************************************************************\
 |                      	FUNCTION DECLARATIONS                      	|
\************************************************************************/
BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, WORD nCmdShow);

LONG FAR PASCAL APIWndProc(HWND hWnd, unsigned msg, WORD wParam, LONG lParam);
LONG FAR PASCAL AboutDlgProc(HWND hDlg, unsigned msg, WORD wParam, LONG lParam);
UINT FAR PASCAL _export WinExecHook(LPSTR lpszCmdLine, UINT fuCmdShow);
HINSTANCE FAR PASCAL _export LoadModHook(LPSTR lpszModuleName,
	LPVOID lpvParamBlk);
HINSTANCE FAR PASCAL _export LoadLibHook(LPSTR lpszModuleName);
HFILE FAR PASCAL _export _lopenHook(LPSTR lpszFilename, int fnOpenMode);
BOOL FAR PASCAL _export WinHelpHook(HWND hWnd, LPSTR lpszHelpFile, UINT fuCmd,
	DWORD dwData);
HFILE FAR PASCAL _export _lcreatHook(LPSTR lpszFilename, int fnOpenMode);
HMODULE FAR PASCAL _export GetModHandleHook(LPSTR lpszModName);
int FAR PASCAL _export SetMapModeHook(HDC hDC, int fnMapMode);
HFILE FAR PASCAL _export OpenFileHook(LPSTR lpszFilename, LPOFSTRUCT lpOpenBuff,
	UINT fuMode);
BOOL FAR PASCAL _export WritePrivProfHook(LPSTR lpszSection, LPSTR lpszEntry,
	LPSTR lpszString, LPSTR lpszFilename);
BOOL FAR PASCAL _export WriteProfHook(LPSTR lpszSection, LPSTR lpszEntry,
	LPSTR lpszString);
DWORD FAR PASCAL _export mciSendStringHook(LPSTR lpszCommand, LPSTR lpszReturn, WORD wRetLen,
	HANDLE hCallback);
DWORD FAR PASCAL _export mciSendCommandHook(WORD wDeviceID, WORD wMessage, DWORD dwParam1,
	DWORD dwParam2);
FARPROC FAR PASCAL _export GetProcAddressHook(HINSTANCE hInstance, LPSTR lpszProcName);
int FAR PASCAL _export OpenPathNameHook(LPSTR lpPathName, int nAccessMode);
int FAR PASCAL _export GetProfileStringHook(LPSTR lpszSection, LPSTR lpszEntry, LPSTR lpszDef,
	LPSTR lpszReturn, int cbReturnBuf);
UINT FAR PASCAL _export GetProfileIntHook(LPSTR lpszSection, LPSTR lpszEntry, int nDef);
int FAR PASCAL _export GetPrivProfileStringHook(LPSTR lpszSection, LPSTR lpszEntry, LPSTR lpszDef,
	LPSTR lpszReturn, int cbReturnBuf, LPSTR lpszFilename);
UINT FAR PASCAL _export GetPrivProfileIntHook(LPSTR lpszSection, LPSTR lpszEntry, int nDef, LPSTR lpszFilename);
ATOM FAR PASCAL _export RegisterClassHook(LPWNDCLASS lpWndCls);

FARPROC SetupHook(LPSTR lpszModule, LPSTR lpszProcName, int nCodeCnt);
void GetTaskModuleName(HANDLE htask, LPSTR modname);
void OutputString(LPSTR lp);
void OutputStringPost(LPSTR lp);

/************************************************************************\
 |                      		DEFINITIONS                        		|
\************************************************************************/
#define MAX_TRAPS					20

#define X_LOADMODULE				0
#define X_WINEXEC	   				1
#define X_LOADLIBRARY				2
#define X__LOPEN					3
#define X__LCREAT					4
#define X_WINHELP					5
#define X_GETMODULEHANDLE			6
#define X_SETMAPMODE				7
#define	X_OPENFILE					8
#define X_WRITEPROFILESTRING		9
#define X_WRITEPRIVATEPROFILESTRING	10
#define X_MCISENDSTRING				11
#define X_MCISENDCOMMAND			12
#define X_GETPROCADDRESS			13
#define X_OPENPATHNAME				14
#define X_GETPROFILESTRING			15
#define X_GETPROFILEINT				16
#define X_GETPRIVATEPROFILESTRING	17
#define X_GETPRIVATEPROFILEINT		18
#define X_REGISTERCLASS				19

typedef DWORD (CALLBACK*      DWFARPROC)();

/************************************************************************\
 |                      		GLOBALS                        			|
\************************************************************************/
FARPROC lpfnHook[MAX_TRAPS];
HANDLE hCapture[MAX_TRAPS];
FARPROC lpfnProlog[MAX_TRAPS];
struct tagGFUNCTIONINFO {
	char	szModule[30];
	char	szProcName[30];
	FARPROC	lpfnHook;
	BOOL	fEnabled;
} gFunctionInfo[MAX_TRAPS] = {
	"KERNEL",	"LoadModule",				(FARPROC)LoadModHook,			FALSE,
	"KERNEL",	"WinExec",					(FARPROC)WinExecHook,			FALSE,
	"KERNEL",	"LoadLibrary",				(FARPROC)LoadLibHook,			FALSE,
	"KERNEL",	"_lopen",					(FARPROC)_lopenHook,			FALSE,
	"KERNEL",	"_lcreat",					(FARPROC)_lcreatHook,			FALSE,
	"USER",		"WinHelp",					(FARPROC)WinHelpHook,			FALSE,
	"KERNEL",	"GetModuleHandle",			(FARPROC)GetModHandleHook,		FALSE,
	"GDI",		"SetMapMode",				(FARPROC)SetMapModeHook,		FALSE,
	"KERNEL",	"OpenFile",					(FARPROC)OpenFileHook,			FALSE,
	"KERNEL",	"WriteProfileString",		(FARPROC)WriteProfHook,			FALSE,
    "KERNEL",	"WritePrivateProfileString",(FARPROC)WritePrivProfHook,		FALSE,
	"MMSYSTEM",	"mciSendString",			(FARPROC)mciSendStringHook,		FALSE,
	"MMSYSTEM",	"mciSendCommand",			(FARPROC)mciSendCommandHook,	FALSE,
    "KERNEL",	"GetProcAddress",			(FARPROC)GetProcAddressHook,	FALSE,
	"KERNEL",	"OpenPathName",				(FARPROC)OpenPathNameHook,		FALSE,
	"KERNEL",	"GetProfileString",			(FARPROC)GetProfileStringHook,	FALSE,
	"KERNEL",	"GetProfileInt",			(FARPROC)GetProfileIntHook,		FALSE,
	"KERNEL",	"GetPrivateProfileString",	(FARPROC)GetPrivProfileStringHook, FALSE,
	"KERNEL",	"GetPrivateProfileInt",		(FARPROC)GetPrivProfileIntHook,	FALSE,
    "USER",		"RegisterClass",			(FARPROC)RegisterClassHook,		FALSE
};

HINSTANCE ghInst;
static HWND ghWndMain, ghWndEdit;

/*
 *	WinMain
 */

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpCmdLine,
	int nCmdShow)
{
	MSG msg;
	char szTmpBuf[128];
	register int x;
    DWORD dwVer;

	dwVer = GetVersion();
	if (LOBYTE(LOWORD(dwVer))!=3 && HIBYTE(LOWORD(dwVer))!=1) {
		MessageBox(NULL, "API Sniffer can only be run under Windows 3.1 RETAIL.", "API Sniffer", MB_ICONINFORMATION);
		return 1;
	}

	ghInst = hInstance;
	if (!hPrevInstance) {
		if (!InitApplication(hInstance))
			return FALSE;
	} else {
		MessageBox(NULL, "API Sniffer is already running.", "API Sniffer", MB_ICONINFORMATION);
		return 1;
    }
	if (!InitInstance(hInstance, nCmdShow))
		return FALSE;

	while (GetMessage(&msg, NULL, NULL, NULL)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

/*
 *	InitApplication
 */

BOOL InitApplication(HANDLE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.lpfnWndProc = (FARPROC)APIWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon(hInstance, "APPICON");
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = COLOR_WINDOW + 1;
	wc.lpszClassName = "APISniff";
	wc.lpszMenuName = "MENU_1";
	return RegisterClass(&wc);
}

/*
 *	InitInstance
 */

BOOL InitInstance(HANDLE hInstance, WORD nCmdShow)
{
	RECT rc;

	ghWndMain  = CreateWindow("APISniff", "API Sniffer", WS_OVERLAPPEDWINDOW | WS_VISIBLE,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL,
		NULL, hInstance, NULL);

	if (!ghWndMain)
		return FALSE;

	GetClientRect(ghWndMain, &rc);
	ghWndEdit = CreateWindow("edit", NULL, WS_CHILD | WS_VISIBLE | ES_MULTILINE |
		WS_VSCROLL | WS_HSCROLL | ES_WANTRETURN, 0, 0, rc.right, rc.bottom, ghWndMain, NULL,
		hInstance, NULL);
	if (!ghWndEdit)
		return FALSE;
	SendMessage(ghWndEdit, WM_SETFONT, GetStockObject(ANSI_FIXED_FONT), NULL);
	SendMessage(ghWndEdit, EM_LIMITTEXT, 60000, 0);

	UpdateWindow(ghWndMain);
	return TRUE;
}

/*
 *	APIWndProc
 *
 *	Description:
 * 		Main window Proc.
 *
 *	Input:		Standard Window Proc.
 *	Outputs:	None.
 */

LONG FAR PASCAL APIWndProc(HWND hWnd, unsigned msg, WORD wParam, LONG lParam)
{
	HMENU hMenu;
	register int x;
	FARPROC lpfn;
	static char szTmpBuf[128];

	switch (msg) {
	case WM_CREATE:
		gFunctionInfo[X_LOADMODULE].fEnabled = GetPrivateProfileInt("APISniff", "LoadModule", 1, "APISNIFF.INI");
		gFunctionInfo[X_WINEXEC].fEnabled = GetPrivateProfileInt("APISniff", "WinExec", 1, "APISNIFF.INI");
		gFunctionInfo[X__LOPEN].fEnabled = GetPrivateProfileInt("APISniff", "_lopen", 1, "APISNIFF.INI");
		gFunctionInfo[X__LCREAT].fEnabled = GetPrivateProfileInt("APISniff", "_lcreat", 1, "APISNIFF.INI");
		gFunctionInfo[X_WINHELP].fEnabled = GetPrivateProfileInt("APISniff", "WinHelp", 1, "APISNIFF.INI");
		gFunctionInfo[X_GETMODULEHANDLE].fEnabled = GetPrivateProfileInt("APISniff", "GetModuleHandle", 0, "APISNIFF.INI");
		gFunctionInfo[X_SETMAPMODE].fEnabled = GetPrivateProfileInt("APISniff", "SetMapMode", 0, "APISNIFF.INI");
		gFunctionInfo[X_OPENFILE].fEnabled = GetPrivateProfileInt("APISniff", "OpenFile", 1, "APISNIFF.INI");
		gFunctionInfo[X_WRITEPROFILESTRING].fEnabled = GetPrivateProfileInt("APISniff", "WriteProfileString", 1, "APISNIFF.INI");
		gFunctionInfo[X_WRITEPRIVATEPROFILESTRING].fEnabled = GetPrivateProfileInt("APISniff", "WritePrivateProfileString", 1, "APISNIFF.INI");
		gFunctionInfo[X_MCISENDSTRING].fEnabled = GetPrivateProfileInt("APISniff", "mciSendString", 1, "APISNIFF.INI");
		gFunctionInfo[X_MCISENDCOMMAND].fEnabled = GetPrivateProfileInt("APISniff", "mciSendCommand", 1, "APISNIFF.INI");
		gFunctionInfo[X_GETPROCADDRESS].fEnabled = GetPrivateProfileInt("APISniff", "GetProcAddress", 1, "APISNIFF.INI");
		gFunctionInfo[X_OPENPATHNAME].fEnabled = GetPrivateProfileInt("APISniff", "OpenPathName", 1, "APISNIFF.INI");
		gFunctionInfo[X_GETPROFILESTRING].fEnabled = GetPrivateProfileInt("APISniff", "GetProfileString", 1, "APISNIFF.INI");
		gFunctionInfo[X_GETPROFILEINT].fEnabled = GetPrivateProfileInt("APISniff", "GetProfileInt", 1, "APISNIFF.INI");
 		gFunctionInfo[X_GETPRIVATEPROFILESTRING].fEnabled = GetPrivateProfileInt("APISniff", "GetPrivateProfileString", 1, "APISNIFF.INI");
		gFunctionInfo[X_GETPRIVATEPROFILEINT].fEnabled = GetPrivateProfileInt("APISniff", "GetPrivateProfileInt", 1, "APISNIFF.INI");
		gFunctionInfo[X_REGISTERCLASS].fEnabled = GetPrivateProfileInt("APISniff", "RegisterClass", 1, "APISNIFF.INI");

		for (x = 0; x < MAX_TRAPS; x++) {
			if (gFunctionInfo[x].fEnabled) {
				// Get an instance thunk to our callback
				lpfnHook[x] = MakeProcInstance(gFunctionInfo[x].lpfnHook, ghInst);

				// Capture it; it will be enabled by default
				hCapture[x] = CaptureSysFunc(gFunctionInfo[x].szModule, gFunctionInfo[x].szProcName, lpfnHook[x], 0);
				if (hCapture[x] == NULL) {
					wsprintf((LPSTR)szTmpBuf, "Capture of %s: %s failed.\r\n", (LPSTR)gFunctionInfo[x].szModule,
						(LPSTR)gFunctionInfo[x].szProcName);
					OutputString(szTmpBuf);
					gFunctionInfo[x].fEnabled = FALSE;
					FreeProcInstance(lpfnHook[x]);
				}                	
            }
		}        
		break;

	case WM_SIZE:
		if (wParam == SIZE_MINIMIZED)
			break;
		MoveWindow(ghWndEdit, 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
		break;

	case WM_COMMAND:
		switch (wParam) {
		case IDM_FILE_EXIT:
			PostMessage(hWnd, WM_CLOSE, NULL, NULL);
			break;

		case IDM_EDIT_COPYALL:
			SendMessage(ghWndEdit, EM_SETSEL, FALSE, MAKELONG(0, -1));
			SendMessage(ghWndEdit, WM_COPY, NULL, NULL);
			SendMessage(ghWndEdit, EM_SETSEL, FALSE, MAKELONG(60000, 60000));
			break;

		case IDM_HELP_ABOUT:
			lpfn = MakeProcInstance((FARPROC)AboutDlgProc, ghInst);
			DialogBox(ghInst, "DIALOG_1", hWnd, lpfn);
			FreeProcInstance(lpfn);
            break;
		}
        break;

	case WM_CLOSE:
		for (x = 0; x < MAX_TRAPS; x++) {
			if (gFunctionInfo[x].fEnabled) {
				ReleaseSysFunc(hCapture[x]);
				FreeProcInstance(lpfnHook[x]);
			}
		}
		PostQuitMessage(0);
		break;

	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

/*
 *	AboutDlgProc
 */

LONG FAR PASCAL AboutDlgProc(HWND hDlg, unsigned msg, WORD wParam, LONG lParam)
{
	switch (msg) {
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		if (wParam == IDOK)
			EndDialog(hDlg, 0);
		break;
	}
    return FALSE;
}

/*
 *	OutputString
 */

void OutputString(LPSTR lp)
{
	SendMessage(ghWndEdit, EM_SETSEL, FALSE, MAKELONG(60000, 60000));
	SendMessage(ghWndEdit, EM_REPLACESEL, NULL, (LONG)lp);
}

/*
 *	OutputStringPost
 */

void OutputStringPost(LPSTR lp)
{
	// We used SendMessage() before, however, when a DOS VM exits, we crash in GetPrivateProfileString().
	// This is thought to be the cause, probably because SendMessage() is doing something internally or
	// because the messages we are sending are causing problems.  So we revert to PostMessage() which
	// will postpone processing till later.
	PostMessage(ghWndEdit, EM_SETSEL, FALSE, MAKELONG(60000, 60000));
	PostMessage(ghWndEdit, EM_REPLACESEL, NULL, (LONG)lp);
}

/*
 *	GetTaskModuleName
 */

void GetTaskModuleName(HANDLE hTask, LPSTR lpModName)
{
    LPSTR lp;
	register int x;

	// The module name resides at 0xF2 in the task database
    lp = (LPSTR)MAKELONG(0xf2, hTask);
	for (x = 0; x < 8; x++)
		*lpModName++ = *lp++;

    *lpModName = '\0';
}

/************************************************************************\
 |                      USER DEFINED HOOKS								|
\************************************************************************/

/*
 *	GetModHandleHook
 */

HMODULE FAR PASCAL _export GetModHandleHook(LPSTR lpszModName)
{
	LPTRAPFUNC lpTF;
	HMODULE hMod;
	static char szTmpBuf[128];

	// Disable the hook so we can call the system function without trapping
	// it.  If we don't disable it, we'll end up right back here!  Notice that
	// this is our first order of business ... this insures that we don't
    // accidently cause a reentrance problem.
	DisableHook(hCapture[X_GETMODULEHANDLE]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: GetModuleHandle(\"%s\")\r\n", (LPSTR)szTmpBuf, lpszModName);
	OutputString((LPSTR)szTmpBuf);

	// CaptureSysFunc returns a handle to memory.  If we lock down this
	// handle and cast it into LPTRAPFUNC, we can reference the actual
	// system address of the hooked function.  Thus, we can use this
	// address to call the system function from within our user-defined
	// hook.
	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_GETMODULEHANDLE]);

	// Make call and be sure to save return value
	hMod = (*lpTF->lpfnSys)(lpszModName);

	// Release the memory
	GlobalUnlock(hCapture[X_GETMODULEHANDLE]);

	// Re-enable the hook so future calls will trap into our user-defined
	// hook
	EnableHook(hCapture[X_GETMODULEHANDLE]);
	return hMod;
}

/*
 *	SetMapModeHook
 */

int FAR PASCAL _export SetMapModeHook(HDC hDC, int fnMapMode)
{
	HDC hRetVal;
	LPTRAPFUNC lpTF;
	static char szTmpBuf[128];
	static char szMode[8][25] = {
		"MM_TEXT",
		"MM_LOMETRIC",
		"MM_HIMETRIC",
		"MM_LOENGLISH",
		"MM_HIENGLISH",
		"MM_TWIPS",
		"MM_ISOTROPIC",
		"MM_ANISOTROPIC"
	};

	DisableHook(hCapture[X_SETMAPMODE]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: SetMapMode(%u, %s)\r\n", (LPSTR)szTmpBuf, hDC, (LPSTR)szMode[fnMapMode - 1]);
	OutputString((LPSTR)szTmpBuf);
    
	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_SETMAPMODE]);
	hRetVal = (*lpTF->lpfnSys)(hDC, fnMapMode);
	GlobalUnlock(hCapture[X_SETMAPMODE]);

	EnableHook(hCapture[X_SETMAPMODE]);
	return hRetVal;
}

/*
 *	WriteProfHook
 */

BOOL FAR PASCAL _export WriteProfHook(LPSTR lpszSection, LPSTR lpszEntry,
	LPSTR lpszString)
{
	BOOL fRetVal;
	static char szTmpBuf[128];
    LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_WRITEPROFILESTRING]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: WriteProfileString(\"%s\", \"%s\", \"%s\")\r\n", (LPSTR)szTmpBuf,
		lpszSection, lpszEntry, lpszString);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_WRITEPROFILESTRING]);
	fRetVal = (*lpTF->lpfnSys)(lpszSection, lpszEntry, lpszString);
	GlobalUnlock(hCapture[X_WRITEPROFILESTRING]);

	EnableHook(hCapture[X_WRITEPROFILESTRING]);
	return fRetVal;
}

/*
 *	WritePrivProfHook
 */

BOOL FAR PASCAL _export WritePrivProfHook(LPSTR lpszSection, LPSTR lpszEntry,
	LPSTR lpszString, LPSTR lpszFilename)
{
	BOOL fRetVal;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_WRITEPRIVATEPROFILESTRING]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: WritePrivateProfileString(\"%s\", \"%s\", \"%s\", \"%s\")\r\n", (LPSTR)szTmpBuf,
		lpszSection, lpszEntry, lpszString, lpszFilename);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_WRITEPRIVATEPROFILESTRING]);
	fRetVal = (*lpTF->lpfnSys)(lpszSection, lpszEntry, lpszString,
		lpszFilename);
	GlobalUnlock(hCapture[X_WRITEPRIVATEPROFILESTRING]);

	EnableHook(hCapture[X_WRITEPRIVATEPROFILESTRING]);
	return fRetVal;
}

/*
 *	OpenFileHook
 */

HFILE FAR PASCAL _export OpenFileHook(LPSTR lpszFilename, LPOFSTRUCT lpOpenBuff,
	UINT fuMode)
{
	HFILE hFile;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_OPENFILE]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);

	// If we are pointing to a bad string, make sure we don't try to print it.
	// 4/15/93 - fixes a bug where exiting a DOS VM would crash system.
	if (IsBadStringPtr(lpszFilename, 20))
		wsprintf((LPSTR)szTmpBuf, "%s: OpenFile( ***BAD STRING POINTER*** )\r\n", (LPSTR)szTmpBuf);
	else
		wsprintf((LPSTR)szTmpBuf, "%s: OpenFile(\"%s\")\r\n", (LPSTR)szTmpBuf, lpszFilename);
	// When exiting a DOS VM, the system would lock up.  Initially thought it was because of the
	// same problem fixed in OpenFileHook().  However, determined that it was because OutputString()
	// was using SendMessage() instead of PostMessage().  Why?  Not sure.  In any case, this code
	// was left behind and is probably not needed.
	OutputStringPost((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_OPENFILE]);
	hFile = (*lpTF->lpfnSys)(lpszFilename, lpOpenBuff, fuMode);
	GlobalUnlock(hCapture[X_OPENFILE]);

	EnableHook(hCapture[X_OPENFILE]);
	return hFile;
}

/*
 *	_lopenHook
 */

HFILE FAR PASCAL _export _lopenHook(LPSTR lpszFilename, int fnOpenMode)
{
	HFILE hFile;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X__LOPEN]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: _lopen(\"%s\")\r\n", (LPSTR)szTmpBuf, lpszFilename);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X__LOPEN]);
	hFile = (*lpTF->lpfnSys)(lpszFilename, fnOpenMode);
	GlobalUnlock(hCapture[X__LOPEN]);

	EnableHook(hCapture[X__LOPEN]);
	return hFile;
}

/*
 *	_lcreatHook
 */

HFILE FAR PASCAL _export _lcreatHook(LPSTR lpszFilename, int fnOpenMode)
{
	HFILE hFile;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X__LCREAT]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: _lcreat(\"%s\")\r\n", (LPSTR)szTmpBuf, lpszFilename);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X__LCREAT]);
	hFile = (*lpTF->lpfnSys)(lpszFilename, fnOpenMode);
	GlobalUnlock(hCapture[X__LCREAT]);

	EnableHook(hCapture[X__LCREAT]);
	return hFile;
}

/*
 *	WinHelpHook
 */

BOOL FAR PASCAL _export WinHelpHook(HWND hWnd, LPSTR lpszHelpFile, UINT fuCmd,
	DWORD dwData)
{
	BOOL fRetVal;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_WINHELP]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: WinHelp(\"%s\")\r\n", (LPSTR)szTmpBuf, lpszHelpFile);
	OutputString((LPSTR)szTmpBuf);


	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_WINHELP]);
	fRetVal = (*lpTF->lpfnSys)(hWnd, lpszHelpFile, fuCmd, dwData);
	GlobalUnlock(hCapture[X_WINHELP]);

	EnableHook(hCapture[X_WINHELP]);
	return fRetVal;
}

/*
 *	WinExecHook
 */

UINT FAR PASCAL _export WinExecHook(LPSTR lpszCmdLine, UINT fuCmdShow)
{
	UINT uRetVal;
	static char szTmpBuf[128];
    LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_WINEXEC]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: WinExec(\"%s\")\r\n", (LPSTR)szTmpBuf, lpszCmdLine);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_WINEXEC]);
	uRetVal = (*lpTF->lpfnSys)(lpszCmdLine, fuCmdShow);
	GlobalUnlock(hCapture[X_WINEXEC]);

	EnableHook(hCapture[X_WINEXEC]);
	return uRetVal;
}

/*
 *	LoadModHook
 */

HINSTANCE FAR PASCAL _export LoadModHook(LPSTR lpszModuleName, LPVOID lpvParamBlk)
{
	HINSTANCE hInstance;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_LOADMODULE]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: LoadModule(\"%s\")\r\n", (LPSTR)szTmpBuf, lpszModuleName);
	OutputString((LPSTR)szTmpBuf);

    lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_LOADMODULE]);
	hInstance = (*lpTF->lpfnSys)(lpszModuleName, lpvParamBlk);
	GlobalUnlock(hCapture[X_LOADMODULE]);

	EnableHook(hCapture[X_LOADMODULE]);
	return hInstance;
}

/*
 *	LoadLibHook
 */

HINSTANCE FAR PASCAL _export LoadLibHook(LPSTR lpszModuleName)
{
	HINSTANCE hInstance;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_LOADLIBRARY]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: LoadLibrary(\"%s\")\r\n", (LPSTR)szTmpBuf, lpszModuleName);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_LOADLIBRARY]);
	hInstance = (*lpTF->lpfnSys)(lpszModuleName);
	GlobalUnlock(hCapture[X_LOADLIBRARY]);

	EnableHook(hCapture[X_LOADLIBRARY]);
	return hInstance;
}

/*
 *	mciSendStringHook
 */

DWORD FAR PASCAL _export mciSendStringHook(LPSTR lpszCommand, LPSTR lpszReturn, WORD wRetLen,
	HANDLE hCallback)
{
	DWORD dwRetVal;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;
	DWFARPROC lpfnSys;

	DisableHook(hCapture[X_MCISENDSTRING]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: mciSendString(\"%s\", ...)\r\n", (LPSTR)szTmpBuf, lpszCommand);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_MCISENDSTRING]);
	lpfnSys = lpTF->lpfnSys;
	dwRetVal = (*lpfnSys)(lpszCommand, lpszReturn, wRetLen, hCallback);
	GlobalUnlock(hCapture[X_MCISENDSTRING]);

	EnableHook(hCapture[X_MCISENDSTRING]);
	return dwRetVal;
}

/*
 *	mciSendCommandHook
 *
 *	This parameters in this function could have been broken down further by dParam1 and then
 *	in cases where necessary, the structure elements pointed to by dParam2 shown.  This wasn't
 *	done here due to time constraints.  This may be done in a future version, however.
 */

DWORD FAR PASCAL _export mciSendCommandHook(WORD wDeviceID, WORD wMessage, DWORD dwParam1,
	DWORD dwParam2)
{
	DWORD dwRetVal;
	static char szTmpBuf[128], szMsg[128];
	LPTRAPFUNC lpTF;
	DWFARPROC lpfnSys;

	DisableHook(hCapture[X_MCISENDCOMMAND]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	switch (wMessage) {
	case MCI_OPEN:
		lstrcpy((LPSTR)szMsg, "MCI_OPEN");
		break;
		    	
	case MCI_CLOSE:
		lstrcpy((LPSTR)szMsg, "MCI_CLOSE");
		break;

	case MCI_ESCAPE:
		lstrcpy((LPSTR)szMsg, "MCI_ESCAPE");
		break;
		    	
	case MCI_PLAY:
		lstrcpy((LPSTR)szMsg, "MCI_PLAY");
		break;
		    	
	case MCI_SEEK:
		lstrcpy((LPSTR)szMsg, "MCI_SEEK");
		break;
		    	
	case MCI_STOP:
		lstrcpy((LPSTR)szMsg, "MCI_STOP");
		break;
		    	
	case MCI_PAUSE:
		lstrcpy((LPSTR)szMsg, "MCI_PAUSE");
		break;
		    	
	case MCI_INFO:
		lstrcpy((LPSTR)szMsg, "MCI_INFO");
		break;
		    	
	case MCI_GETDEVCAPS:
		lstrcpy((LPSTR)szMsg, "MCI_GETDEVCAPS");
		break;
		    	
	case MCI_SPIN:
		lstrcpy((LPSTR)szMsg, "MCI_SPIN");
		break;
		    	
	case MCI_SET:
		lstrcpy((LPSTR)szMsg, "MCI_SET");
		break;
		    	
	case MCI_STEP:
		lstrcpy((LPSTR)szMsg, "MCI_STEP");
		break;
		    	
	case MCI_RECORD:
		lstrcpy((LPSTR)szMsg, "MCI_RECORD");
		break;

	case MCI_SYSINFO:
		lstrcpy((LPSTR)szMsg, "MCI_SYSINFO");
		break;

	case MCI_BREAK:
		lstrcpy((LPSTR)szMsg, "MCI_BREAK");
		break;
		    	
	case MCI_SOUND:
		lstrcpy((LPSTR)szMsg, "MCI_SOUND");
		break;
		    	
	case MCI_SAVE:
		lstrcpy((LPSTR)szMsg, "MCI_SAVE");
		break;
		    	
	case MCI_STATUS:
		lstrcpy((LPSTR)szMsg, "MCI_STATUS");
		break;
		    	
	case MCI_CUE:
		lstrcpy((LPSTR)szMsg, "MCI_CUE");
		break;
		    	
	case MCI_REALIZE:
		lstrcpy((LPSTR)szMsg, "MCI_REALIZE");
		break;
		    	
	case MCI_WINDOW:
		lstrcpy((LPSTR)szMsg, "MCI_WINDOW");
		break;
		    	
	case MCI_PUT:
		lstrcpy((LPSTR)szMsg, "MCI_PUT");
		break;
		    	
	case MCI_WHERE:
		lstrcpy((LPSTR)szMsg, "MCI_WHERE");
		break;
		    	
	case MCI_FREEZE:
		lstrcpy((LPSTR)szMsg, "MCI_FREEZE");
		break;
		    	
	case MCI_UNFREEZE:
		lstrcpy((LPSTR)szMsg, "MCI_UNFREEZE");
		break;
		    	
	case MCI_LOAD:
		lstrcpy((LPSTR)szMsg, "MCI_LOAD");
		break;
		    	
	case MCI_CUT:
		lstrcpy((LPSTR)szMsg, "MCI_CUT");
		break;
		    	
	case MCI_COPY:
		lstrcpy((LPSTR)szMsg, "MCI_COPY");
		break;
		    	
	case MCI_PASTE:
		lstrcpy((LPSTR)szMsg, "MCI_PASTE");
		break;
		    	
	case MCI_UPDATE:
		lstrcpy((LPSTR)szMsg, "MCI_UPDATE");
		break;
		    	
	case MCI_RESUME:
		lstrcpy((LPSTR)szMsg, "MCI_RESUME");
		break;
		    	
	case MCI_DELETE:
		lstrcpy((LPSTR)szMsg, "MCI_DELETE");
		break;

    default:
		wsprintf((LPSTR)szMsg, "%u", wMessage);
		break;
    }
	wsprintf((LPSTR)szTmpBuf, "%s: mciSendCommand(%u, %s, %lu, %lu)\r\n", (LPSTR)szTmpBuf, wDeviceID,
		(LPSTR)szMsg, dwParam1, dwParam2);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_MCISENDCOMMAND]);

	// FARPROC is designed to return an INT.  We use our DWFARPROC which will return a DWORD.
	lpfnSys = lpTF->lpfnSys;
	dwRetVal = (*lpfnSys)(wDeviceID, wMessage, dwParam1, dwParam2);
	GlobalUnlock(hCapture[X_MCISENDCOMMAND]);

	EnableHook(hCapture[X_MCISENDCOMMAND]);
	return dwRetVal;
}

/*
 *	GetProcAddress
 */

FARPROC FAR PASCAL _export GetProcAddressHook(HINSTANCE hInstance, LPSTR lpszProcName)
{
	FARPROC lpfn;
	static char szTmpBuf[128], szTmpBuf2[65], szModName[65];
	LPTRAPFUNC lpTF;
	DWFARPROC lpfnSys;

	DisableHook(hCapture[X_GETPROCADDRESS]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	GetModuleFileName(hInstance, (LPSTR)szModName, sizeof(szModName));
    szTmpBuf2[64] = NULL;
	if (IsBadStringPtr(lpszProcName, 64))
		wsprintf((LPSTR)szTmpBuf, "%s: GetProcAddress( %u (%s), ***BAD STRING POINTER*** )\r\n", (LPSTR)szTmpBuf,
			hInstance, (LPSTR)szModName);
	else {
    	lstrcpyn((LPSTR)szTmpBuf2, lpszProcName, 64);
		wsprintf((LPSTR)szTmpBuf, "%s: GetProcAddress(%u (%s), \"%s\")\r\n", (LPSTR)szTmpBuf,
			hInstance, (LPSTR)szModName, (LPSTR)szTmpBuf2);
	}
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_GETPROCADDRESS]);
    lpfnSys = lpTF->lpfnSys;
	lpfn = (*lpfnSys)(hInstance, lpszProcName);
	GlobalUnlock(hCapture[X_GETPROCADDRESS]);

	EnableHook(hCapture[X_GETPROCADDRESS]);
	return lpfn;
}

/*
 *	OpenPathNameHook (undocumented)
 */

int FAR PASCAL _export OpenPathNameHook(LPSTR lpszPathName, int nAccessMode)
{
	static char szTmpBuf[128], szTmpBuf2[65];
	LPTRAPFUNC lpTF;
	int n;

	DisableHook(hCapture[X_OPENPATHNAME]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
    szTmpBuf2[64] = NULL;
	if (IsBadStringPtr(lpszPathName, 64))
		wsprintf((LPSTR)szTmpBuf, "%s: OpenPathName( ***BAD STRING POINTER*** )\r\n", (LPSTR)szTmpBuf);
	else {
		lstrcpyn((LPSTR)szTmpBuf2, lpszPathName, 64);
		wsprintf((LPSTR)szTmpBuf, "%s: OpenPathName(\"%s\", %d)\r\n", (LPSTR)szTmpBuf, (LPSTR)szTmpBuf2,
			nAccessMode);
	}
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_OPENPATHNAME]);
	n = (*lpTF->lpfnSys)(lpszPathName, nAccessMode);
	GlobalUnlock(hCapture[X_OPENPATHNAME]);

	EnableHook(hCapture[X_OPENPATHNAME]);
	return n;
}

/*
 *	GetProfileStringHook
 */

int FAR PASCAL _export GetProfileStringHook(LPSTR lpszSection, LPSTR lpszEntry, LPSTR lpszDef,
	LPSTR lpszReturn, int cbReturnBuf)
{
	int n;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_GETPROFILESTRING]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: GetProfileString(\"%s\", \"%s\", \"%s\", ...)\r\n", (LPSTR)szTmpBuf,
		lpszSection, lpszEntry, lpszDef);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_GETPROFILESTRING]);
	n = (*lpTF->lpfnSys)(lpszSection, lpszEntry, lpszDef, lpszReturn, cbReturnBuf);
	GlobalUnlock(hCapture[X_GETPROFILESTRING]);

	EnableHook(hCapture[X_GETPROFILESTRING]);
	return n;
}

/*
 *	GetProfileIntHook
 */

UINT FAR PASCAL _export GetProfileIntHook(LPSTR lpszSection, LPSTR lpszEntry, int nDef)
{
	UINT u;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_GETPROFILEINT]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: GetProfileInt(\"%s\", \"%s\", %d)\r\n", (LPSTR)szTmpBuf,
		lpszSection, lpszEntry, nDef);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_GETPROFILEINT]);
	u = (*lpTF->lpfnSys)(lpszSection, lpszEntry, nDef);
	GlobalUnlock(hCapture[X_GETPROFILEINT]);

	EnableHook(hCapture[X_GETPROFILEINT]);
	return u;
}

/*
 *	GetPrivProfileStringHook
 */

int FAR PASCAL _export GetPrivProfileStringHook(LPSTR lpszSection, LPSTR lpszEntry, LPSTR lpszDef,
	LPSTR lpszReturn, int cbReturnBuf, LPSTR lpszFilename)
{
	int n;
	static char szTmpBuf[256], szDef[65], szFile[128], szSection[65], szEntry[65];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_GETPRIVATEPROFILESTRING]);

	// When exiting a DOS VM, the system would lock up.  Initially thought it was because of the
	// same problem fixed in OpenFileHook().  However, determined that it was because OutputString()
	// was using SendMessage() instead of PostMessage().  Why?  Not sure.  In any case, this code
	// was left behind and is probably not needed.
	szDef[64] = szFile[127] = szSection[64] = szEntry[64] = NULL;
	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	if (IsBadStringPtr(lpszDef, 64))
		wsprintf((LPSTR)szDef, " ***BAD STRING POINTER*** ");
	else
		lstrcpyn((LPSTR)szDef, lpszDef, 64);
	if (IsBadStringPtr(lpszFilename, 127))
		wsprintf((LPSTR)szFile, " ***BAD STRING POINTER*** ");
	else
		lstrcpyn((LPSTR)szFile, lpszFilename, 127);
	if (IsBadStringPtr(lpszSection, 64))
		wsprintf((LPSTR)szSection, " ***BAD STRING POINTER*** ");
	else
		lstrcpyn((LPSTR)szSection, lpszSection, 64);
	if (IsBadStringPtr(lpszEntry, 64))
		wsprintf((LPSTR)szEntry, " ***BAD STRING POINTER*** ");
	else
		lstrcpyn((LPSTR)szEntry, lpszEntry, 64);

	wsprintf((LPSTR)szTmpBuf, "%s: GetPrivateProfileString(%s\", \"%s\", \"%s\", ..., \"%s\")\r\n", (LPSTR)szTmpBuf,
		(LPSTR)szSection, (LPSTR)szEntry, (LPSTR)szDef, (LPSTR)szFile);
	OutputStringPost((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_GETPRIVATEPROFILESTRING]);
	n = (*lpTF->lpfnSys)(lpszSection, lpszEntry, lpszDef, lpszReturn, cbReturnBuf, lpszFilename);
	GlobalUnlock(hCapture[X_GETPRIVATEPROFILESTRING]);

	EnableHook(hCapture[X_GETPRIVATEPROFILESTRING]);
	return n;
}

/*
 *	GetPrivProfileIntHook
 */

UINT FAR PASCAL _export GetPrivProfileIntHook(LPSTR lpszSection, LPSTR lpszEntry, int nDef, LPSTR lpszFilename)
{
	UINT u;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_GETPRIVATEPROFILEINT]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: GetPrivateProfileInt(\"%s\", \"%s\", %d, \"%s\")\r\n", (LPSTR)szTmpBuf,
		lpszSection, lpszEntry, nDef, lpszFilename);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_GETPRIVATEPROFILEINT]);
	u = (*lpTF->lpfnSys)(lpszSection, lpszEntry, nDef, lpszFilename);
	GlobalUnlock(hCapture[X_GETPRIVATEPROFILEINT]);

	EnableHook(hCapture[X_GETPRIVATEPROFILEINT]);
	return u;
}

/*
 *	RegisterClassHook
 */

ATOM FAR PASCAL _export RegisterClassHook(LPWNDCLASS lpWndCls)
{
	ATOM a;
	static char szTmpBuf[128];
	LPTRAPFUNC lpTF;

	DisableHook(hCapture[X_REGISTERCLASS]);

	GetTaskModuleName(GetCurrentTask(), (LPSTR)szTmpBuf);
	wsprintf((LPSTR)szTmpBuf, "%s: RegisterClass(lpszClassName=\"%s\", ...)\r\n", (LPSTR)szTmpBuf,
    	lpWndCls->lpszClassName);
	OutputString((LPSTR)szTmpBuf);

	lpTF = (LPTRAPFUNC)GlobalLock(hCapture[X_REGISTERCLASS]);
	a = (*lpTF->lpfnSys)(lpWndCls);
	GlobalUnlock(hCapture[X_REGISTERCLASS]);

	EnableHook(hCapture[X_REGISTERCLASS]);
	return a;
}
